//----------------------------------------------------------------------------
// TAviDialog v. 1.0 - (C) Copyright 1995 by Kent Reisdorph, 75522,1174
//
// July 25, 1995
//
// #include "avidlg.h"
//
// Code for class TAviDialog, a class to play an AVI video in
// a 3D dialog box with start, stop, rewind buttons and a slider
// control.
//
// Implementation: TAviDialog(TWindow* parent, const char far* filename,
//				const char far* title, BOOL autoplay, BOOL autoclose,
//				TModule* module)
//
// Example: TAviDialog(this, "some.avi");
//----------------------------------------------------------------------------

#include "avidlg.h"

DEFINE_RESPONSE_TABLE1(TAviDialog, TDialog)
	EV_WM_TIMER,
	EV_WM_HSCROLL,
	EV_WM_PAINT,
	EV_COMMAND(IDC_PLAY, CmPlay),
	EV_COMMAND(IDC_STOP, CmStop),
	EV_COMMAND(IDC_REWIND, CmRewind),
END_RESPONSE_TABLE;

// Constructor. title can be used to pass a string to be used as a
// caption. If title is not specified then the filename will be used
// as the caption. filename is the avi filename to be played. autoplay
// indicates whether the video should start playing as soon as the
// dialog becomes visible. autoclose specifies whether the dialog should
// close when the video has finished playing. Sets up owner drawn buttons,
// a static control to use as a frame for the slider, a frame around the
// video, and a static control that we will use as the playback window.
// Also, we check to see if Ctl3d is enabled so that we can reset the
// calling app back the way we found it.
TAviDialog::TAviDialog(TWindow* parent, const char far* filename,
				const char far* title, BOOL autoplay, BOOL autoclose,
				TModule* module)
	: TDialog(parent, IDD_PLAYER, module)
{
	fileName = filename;
	autoPlay = autoplay;
	autoClose = autoclose;
	if (strlen(title)>0) SetCaption(title);
	else {
		strupr((char*)filename);
		SetCaption(filename);
	}
	new TODAButton(this, IDC_PLAY, IDB_BTNUP, IDB_PLAY, module);
	new TODAButton(this, IDC_STOP, IDB_BTNUP, IDB_STOP, module);
	new TODAButton(this, IDC_REWIND, IDB_BTNUP, IDB_REWIND, module);
	VideoWindow = new TStatic(this, IDC_VIDEO);
	Frame = new TStatic(this, IDC_FRAME);
	SliderStatic = new TStatic(this, IDC_SLIDER);
	ctl3dEnabled = GetApplication()->Ctl3dEnabled();
	if (!ctl3dEnabled) GetApplication()->EnableCtl3d(TRUE);
}

// Destructor. Closes MCI device. This is necessary so that we don't
// leave the device driver hanging in memory. Also, we disable Ctl3d
// if the calling app did not have it enabled.
TAviDialog::~TAviDialog()
{
	mciSendCommand(deviceID, MCI_CLOSE, 0, NULL);
	if (!ctl3dEnabled) GetApplication()->EnableCtl3d(FALSE);
}

// SetupWindow. Lots going on here but most of it is handled in
// SizeDialog(). If autoplay == TRUE then start playback
void
TAviDialog::SetupWindow()
{
	TDialog::SetupWindow();
	if (OpenFile()) Destroy();
	SizeDialog();
	if (autoPlay) CmPlay();
}

// EvTimer. Gets the current avi position and sets the position of the
// slider accordingly. Shuts down the dialog if autoClose == TRUE and if
// the end of the video has been reached.
void
TAviDialog::EvTimer(UINT)
{
	if (GetAviPosition() >= fileLength) {
		KillTimer(1);
		if (autoClose) CloseWindow();
	}
	Slider->SetPosition((int)GetAviPosition());
}

// EvPaint. This is required so that the video frame is repainted when
// a WM_PAINT is sent to the dialog. We only need to repaint if the video
// is stopped.
void
TAviDialog::EvPaint()
{
	TDialog::EvPaint();
	if (GetAviStatus(MCI_STATUS_MODE) == MCI_MODE_STOP) PaintAvi();
}

// EvHScroll. Sets the AVI frame to the current value of the scroller
// if the scroller is being dragged.
void
TAviDialog::EvHScroll(UINT scrollCode, UINT thumbPos, HWND hWndCtl)
{
	TDialog::EvHScroll(scrollCode, thumbPos, hWndCtl);
	SetAviPosition(Slider->GetPosition());
}

// CmPlay. Use MCI_PLAY to begin playback. MCI_NOTIFY is used as the flag
// in the mciSendCommand so that control returns back to the dialog. We
// must have this so that the stop and rewind buttons function during
// playback. If we use MCI_WAIT then everything comes to a halt until
// the video is done playing. Set a timer so that the scroller gets
// updated.
void
TAviDialog::CmPlay()
{
	SetTimer(1,50);
	MCI_PLAY_PARMS Play;
	Play.dwCallback = MAKELONG(HWindow, 0); // not used at present
	mciSendCommand(deviceID, MCI_PLAY, MCI_NOTIFY, (DWORD)(LPVOID) &Play);
}

// CmStop. Stops the video using MCI_STOP if the stop button was pressed
// and kills the timer.
void
TAviDialog::CmStop()
{
	MCI_GENERIC_PARMS Generic;
	Generic.dwCallback = MAKELONG(HWindow, 0); // not used at present
	mciSendCommand(deviceID, MCI_STOP, MCI_NOTIFY, (DWORD)(LPVOID) &Generic);
	KillTimer(1);
}

// CmRewind. Sets the video position to 0 if the stop button was pressed,
// kills the timer in case it's still running, and sets the slider to 0.
void
TAviDialog::CmRewind()
{
	SetAviPosition(0);
	Slider->SetPosition(0);
	KillTimer(1);
}

// SetAviPosition. Uses MCI_SEEK to set the video frame to the value
// specified in to. CmRewind uses this function to set the current frame
// to 0 and EvHScroll uses it to set the video frame to the value of the
// slider control.
void
TAviDialog::SetAviPosition(DWORD to)
{
	DWORD flags = MCI_SEEK_TO_START;
	if (to) flags = MCI_TO | MCI_WAIT;
	MCI_SEEK_PARMS Seek;
	Seek.dwCallback = MAKELONG(HWindow, 0); // not used at present
	Seek.dwTo = to;
	mciSendCommand(deviceID, MCI_SEEK, flags, (DWORD)(LPVOID) &Seek);
}

// GetAviStatus. Uses MCI_STATUS with the MCI_STATUS_ITEM to get a
// variety of status queries.
DWORD
TAviDialog::GetAviStatus(DWORD item)
{
	MCI_STATUS_PARMS Status;
	//Status.dwCallback = MAKELONG(hWindow, 0);
	Status.dwItem = item;
	mciSendCommand(deviceID, MCI_STATUS, MCI_STATUS_ITEM, (DWORD)(LPVOID) &Status);
	return Status.dwReturn;
}

// OpenFile. Opens the video device and the video file specified in the
// constructor. Passes 0 as the device ID. After opening, the wDeviceID
// member of MCI_OPEN_PARMS contains the device ID we use on subsequent
// mci calls.
DWORD
TAviDialog::OpenFile()
{
	DWORD flags = MCI_OPEN_ELEMENT | MCI_OPEN_TYPE;
	MCI_OPEN_PARMS Open;
	Open.lpstrDeviceType = "avivideo";
	Open.lpstrElementName = fileName;
	DWORD result = mciSendCommand(0, MCI_OPEN, flags, (DWORD)(LPVOID) &Open);
	if (!result) {
		deviceOpen = TRUE;
		deviceID = Open.wDeviceID;
	}
	return result;
}

// PaintAvi. Uses MCI_UPDATE to repaint the video window. Uses the DC
// from the static we are using for the playback window. We use the API
// GetDC rather than a TDC because we only need it for a sec.
void
TAviDialog::PaintAvi()
{
	MCI_DGV_UPDATE_PARMS Parms;
	Parms.hDC = GetDC(playbackWindow);
	Parms.rc = Where(MCI_WAIT | MCI_DGV_WHERE_VIDEO);
	mciSendCommand(deviceID, MCI_UPDATE, MCI_DGV_UPDATE_HDC, (DWORD)(LPVOID)&Parms);
	ReleaseDC(playbackWindow, Parms.hDC);
}

// This function tells MCI to set the size of the video to the size of
// the static control VideoWindow
DWORD
TAviDialog::SetDestinationRect(int x, int y, int w, int h, DWORD flags)
{
	flags |= MCI_DGV_PUT_DESTINATION | MCI_DGV_RECT;
	RECT rect;
	rect.left = x;
	rect.top = y;
	rect.right = x + w;
	rect.bottom = y + h;
	MCI_DGV_PUT_PARMS Parms;
	if ( flags |= MCI_NOTIFY ) Parms.dwCallback = MAKELONG(HWindow, 0);
	Parms.rc = rect;
	return mciSendCommand(deviceID, MCI_PUT, flags, (DWORD)(LPVOID) &Parms);
}

// Where. Uses MCI_WHERE to return the size of the video rect via the
// inline function GetDestinationRect so that the entire dialog can be
// sized to the size of the avi. Also used by PaintAvi to obtain the
// rect to be painted.
RECT
TAviDialog::Where(DWORD flags)
{
	MCI_DGV_WHERE_PARMS Parms;
	if ( flags |= MCI_NOTIFY ) Parms.dwCallback = MAKELONG(HWindow, 0);
	mciSendCommand(deviceID, MCI_WHERE, flags, (DWORD)(LPVOID) &Parms);
	return Parms.rc;
}

// Window. Uses MCI_WINDOW to set the playback window to the static
// control, VideoWindow.
DWORD
TAviDialog::SetPlaybackWindow(HWND hwnd)
{
	playbackWindow = hwnd;
	MCI_DGV_WINDOW_PARMS Parms;
	DWORD flags = MCI_DGV_WINDOW_STATE | MCI_DGV_WINDOW_HWND;
	Parms.nCmdShow = SW_NORMAL;
	Parms.hWnd = hwnd;
	return mciSendCommand(deviceID, MCI_WINDOW, flags,
							(DWORD)(LPVOID) &Parms);
}

// This does all the sizing and positioning of the various controls.
// It also sizes the dialog accordingly. In order to achieve the 3D
// look I have a frame (Frame) around the static control that holds
// the video window (VideoWindow). The slider is set to the width of
// the video window, and the buttons are centered beneath.
void
TAviDialog::SizeDialog()
{
	int frameOffset = 7;
	int CyCaption = GetSystemMetrics(SM_CYCAPTION);
	int RightMargin = CyCaption + frameOffset;
	int BottomMargin = CyCaption * 4;
	SetPlaybackWindow(VideoWindow->HWindow);
	// Set the slider range
	// Get the size of the Video Image
	TRect vdRect = GetSourceRect();
	// Get the size of the outside frame
	TRect frameRect = Frame->GetWindowRect();
	TPoint UlPoint;
	// Get the window rect
	TRect dgRect = GetWindowRect();
	UlPoint.x = dgRect.left + CyCaption;
	UlPoint.y = dgRect.top + (CyCaption * 2);
	TPoint LrPoint = vdRect.BottomRight() + UlPoint;
	// Calculate a new dialog width and height based on the width
	// and height of the AVI window plus a margin. The bottom margin
	// has to leave room for the slider and buttons.
	int x1 = UlPoint.x;
	int x = UlPoint.x + vdRect.Width() + RightMargin;
	int y1 = UlPoint.y + vdRect.Height();
	int y = y1 + BottomMargin;
	TPoint dgPoint = dgRect.TopLeft();
	dgRect.Set(dgPoint.x, dgPoint.y, x, y);
	MoveWindow(dgRect);
	// Expand the static to 6 points larger than the video rect
	LrPoint.Offset(frameOffset, frameOffset);
	// Convert to Dialog units
	ScreenToClient(UlPoint);
	ScreenToClient(LrPoint);
	// Now we have the size of the outside frame
	frameRect = TRect(UlPoint, LrPoint);
	// So we can size it
	Frame->MoveWindow(frameRect);
	// The inside frame is 2 points smaller than the outside frame
	VideoWindow->MoveWindow(frameRect.InflatedBy(-2, -2));
	// Put the video in the static but offset by 1 pixel so that
	// it doesn't cover any of the inside frame.
	SetDestinationRect(1, 1, vdRect.Width() - 1, vdRect.Height() - 1);
	// Next create the slider
	UlPoint = TPoint(x1, y1 + CyCaption/2);
	ScreenToClient(UlPoint);
	Slider = new TPlainSlider(this, IDC_SLIDER,
		UlPoint.x, UlPoint.y,	frameRect.Width(), CyCaption);
	Slider->Create();
	fileLength = GetAviLength();
	Slider->SetRange(1, (int)fileLength);
	Slider->SetRuler(0);
	Slider->SetPosition(1);
	// Move the buttons. This is set up so that you can add
	// more buttons if you choose. Button Ids start at 100
	// and must be sequential.
	int numBtns = 3;
	TRect rect;
	for (int i=0;i<numBtns;i++) {
		// get a handle for the first button
		HWND hWnd = GetDlgItem(i + 101);
		// get the button rect
		::GetWindowRect(hWnd, &rect);
		// get the width of the set of buttons
		int totalWidth = (numBtns * (rect.Width() + 2));
		// calculate an X start point
		int offset = ((dgRect.Width()/2) - (totalWidth/2)) - 4;
		// add a button width + a space to the offset
		offset += (( i * rect.Width() ) + ( i * 2));
		// only the Y point needs to be converted to dialog units
		// the offset is already in dialog units.
		UlPoint = TPoint(0, (y1 + (CyCaption * 2)));
		ScreenToClient(UlPoint);
		// move the button
		::MoveWindow(hWnd, offset, UlPoint.y, rect.Width(), rect.Height(), 0);
	}
}

// The following is the owner drawn button class.
DEFINE_RESPONSE_TABLE2(TODAButton, TButton, TControl)
  EV_WM_MOUSEMOVE,
  EV_MESSAGE(BM_SETSTYLE, BMSetStyle),
END_RESPONSE_TABLE;

void
TODAButton::SetupWindow()
{
	TButton::SetupWindow();
	HINSTANCE HInstance = GetModule()->GetInstance();
	TRect rct = GetWindowRect();
	TPoint ulpoint = rct.TopLeft();
	TBitmap* bm = new TBitmap(HInstance, bmId);
	TPoint lrpoint(ulpoint.x + (bm->Width()+2), ulpoint.y + (bm->Height()+2));
	Parent->ScreenToClient(ulpoint);
	Parent->ScreenToClient(lrpoint);
	MoveWindow(ulpoint.x, ulpoint.y,
							lrpoint.x - ulpoint.x, lrpoint.y - ulpoint.y);
	delete bm;
}

LRESULT
TODAButton::BMSetStyle(WPARAM wParam, LPARAM)
{
	if ((Attr.Style & BS_OWNERDRAW) != BS_OWNERDRAW)
	 DefaultProcessing();

	else if (wParam == BS_DEFPUSHBUTTON) {
	 IsCurrentDefPB = TRUE;
	 Invalidate(FALSE);

	} else if (wParam == BS_PUSHBUTTON) {
		IsCurrentDefPB = FALSE;
	 Invalidate(FALSE);

	} else {
		DefaultProcessing();
  }

  return 0;
}

void
TODAButton::ODADrawEntire(DRAWITEMSTRUCT far& drawInfo)
{
	TDC dc(drawInfo.hDC);
	HINSTANCE HInstance = GetModule()->GetInstance();
	TMemoryDC memdc(dc);
	TBitmap* bmBtnUp = new TBitmap(HInstance, bmId);
	TBitmap* bmBtnDn = new TBitmap(HInstance, bmId + 1);
	TBitmap* bmBtnFace = new TBitmap(HInstance, bm2Id);
	TRect rect(drawInfo.rcItem.left, drawInfo.rcItem.top,
								 drawInfo.rcItem.right, drawInfo.rcItem.bottom);
	TRect bmRect(drawInfo.rcItem.left+1, drawInfo.rcItem.top+1,
								 drawInfo.rcItem.right, drawInfo.rcItem.bottom);
	TBrush* brush;
	if (drawInfo.itemState & ODS_FOCUS)
		brush = new TBrush(TColor::Black);
	else
		brush = new TBrush(TColor::LtGray);
	dc.SelectObject(*brush);
	dc.Rectangle(rect.InflatedBy(1, 1));
	if (drawInfo.itemState & ODS_SELECTED) {
		memdc.SelectObject(*bmBtnDn);
		dc.BitBlt(bmRect, memdc, TPoint(0,0), SRCCOPY);
		memdc.SelectObject(*bmBtnFace);
		dc.BitBlt(bmRect.OffsetBy(3,3), memdc, TPoint(0,0), SRCCOPY);
	}
	else {
		memdc.SelectObject(*bmBtnUp);
		dc.BitBlt(bmRect, memdc, TPoint(0,0), SRCCOPY);
		memdc.SelectObject(*bmBtnFace);
		dc.BitBlt(bmRect.OffsetBy(2,2), memdc, TPoint(0,0), SRCCOPY);
	}
	delete brush;
	delete bmBtnUp;
	delete bmBtnDn;
	if (bmBtnFace) delete bmBtnFace;
}

